package org.semanticweb.glp;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.HashSet;
import org.semanticweb.lp.Term;
import org.semanticweb.lp.VariableTerm;

/**
 *  Title: Triple Description: RDF Query and Inference Language Copyright:
 *  Copyright (c) 2001 Company: Stanford University
 *
 *@author     Stefan Decker
 *@created    January 17, 2002
 *@version    1.0
 */

public class GLPTermVariable extends GLPTerm {
    /**
     *  Description of the Field
     */
    public String v;

    /** create anonymous GLPTermVariable [MS]
     */
    public GLPTermVariable() {
        this.v = "_"; // hack at the moment ???
    }
    
    /**
     *  Constructor for the GLPTermVariable object
     *
     *@param  v  Description of the Parameter
     */
    public GLPTermVariable(String v) {
        this.v = v;
    }


    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    public Term toLPTerm() {
        return new VariableTerm(v);
    }


    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    public GLPTerm copyTerm() {
        return new GLPTermVariable(v);
    }


    /**
     *  Description of the Method
     *
     *@param  h  Description of the Parameter
     *@return    Description of the Return Value
     */
    public GLPTerm copyTerm(HashMap h) {
        GLPTerm copy;
        GLPTermVariable n = (GLPTermVariable) h.get(this);
        if (n != null) {
            copy = n;
        }
        else {
            copy = new GLPTermVariable(v);
        }
        return copy;
    }


    /**
     *  Description of the Method
     *
     *@param  free   Description of the Parameter
     *@param  bound  Description of the Parameter
     */
    public void freeBoundVariables(HashSet free, HashSet bound) {
        if (!bound.contains(this) && !free.contains(this)) {
            free.add(this);
        }
    }


    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    public String toString() {
        return v;
    }


    /**
     *  Description of the Method
     *
     *@return    Description of the Return Value
     */
    public int hashCode() {
        return v.hashCode();
    }


    /**
     *  Description of the Method
     *
     *@param  o  Description of the Parameter
     *@return    Description of the Return Value
     */
    public boolean equals(Object o) {
        if (!(o instanceof GLPTermVariable)) {
            return false;
        }
        else {
            return toString().equals(o.toString());
        }
    }


    /**
     *  Description of the Method
     *
     *@param  h  Description of the Parameter
     */
    public void renameVariables(HashMap h) {
        GLPTermVariable newVariable = (GLPTermVariable) h.get(this);
        if (newVariable != null) {
            v = newVariable.toString();
        }
    }
    
}
