package org.semanticweb.lp;

import java.util.ArrayList;
import java.util.Iterator;
import java.io.*;
import org.semanticweb.utilities.ASCII_UCodeESC_CharStream;

/**
 *  Description of the Class
 *
 *@author     Administrator
 *@created    January 17, 2002
 */
public class LPParser {
    char c, d;
    boolean fini;
    ASCII_UCodeESC_CharStream is;
    LPParserCallBack callback;


    /**
     *  Constructor for the SimpleParser object
     *
     *@param  stream    Description of the Parameter
     *@param  callback  Description of the Parameter
     */
    public LPParser(InputStream stream, LPParserCallBack callback) {
        this.callback = callback;
        is = new ASCII_UCodeESC_CharStream(stream, 1, 1);
    }


    /**
     *  Description of the Method
     *
     *@return                       Description of the Return Value
     *@exception  LPParseException  Description of the Exception
     */
    final boolean arrow() throws LPParseException {
        if ((c == '<') || (c == ':')) {
            try {
                d = is.readChar();
            } catch (java.io.IOException e) {
                throw new LPParseException("Line:" + is.getEndLine() + ".Column:" + is.getEndColumn() + ". Encountered  " + d + ". Unexpected End of File");
            }
            if (d == '-') {
                skip();
                return true;
            }
            else {
                is.backup(1);
                return false;
            }
        }
        else {
            return false;
        }
    }


    /**
     *  Description of the Method
     *
     *@return                       Description of the Return Value
     *@exception  LPParseException  Description of the Exception
     */
    final Literal bodyLiteral() throws LPParseException {
        Term[] args = null;
        String s;
        int code;
        boolean neg = false;
        if (c == '-') {
            neg = true;
            skip();
        }
        s = identifier();
        if (c == '(') {
            skip();
            args = parameterList();
            bracketRight();
        }
        return new Literal(s, args);
    }


    /**
     *  Description of the Method
     *
     *@return                       Description of the Return Value
     *@exception  LPParseException  Description of the Exception
     */
    final Literal[] bodyConjunction() throws LPParseException {
        Literal body[];
        ArrayList bodies = new ArrayList(10);
        bodies.add(bodyLiteral());
        while ((c == '&') || (c == ',')) {
            skip();
            bodies.add(bodyLiteral());
        }
        int l = bodies.size();
        body = new Literal[l];

        //Sort Body Atoms (positive first, negative last)
        int k = 0;
        int m = l - 1;
        for (Iterator i = bodies.iterator(); i.hasNext(); ) {
            Literal lit = (Literal) i.next();
            if (lit.isAtom()) {
                body[k] = lit;
                k++;
            }
            else {
                body[m] = lit;
                m--;
            }
        }
        return body;
    }


    /**
     *  Description of the Method
     *
     *@return                       Description of the Return Value
     *@exception  LPParseException  Description of the Exception
     */
    final boolean bracketLeft() throws LPParseException {
        if (c != '(') {
            return false;
        }
        else {
            skip();
            return true;
        }
    }


    /**
     *  Description of the Method
     *
     *@exception  LPParseException  Description of the Exception
     */
    final void bracketRight() throws LPParseException {
        if (c != ')') {
            throw new LPParseException("Line:" + is.getEndLine() + ".Column:" + is.getEndColumn() + ". Encountered  " + c + ". Closing Bracket expected");
        }
        skip();
    }


    /**
     *  Description of the Method
     *
     *@exception  LPParseException  Description of the Exception
     */
    public final void parse() throws LPParseException {
        skip();
        while (!fini) {
            callback.call(parseRule());
        }
        is.Done();
    }


    /**
     *  Description of the Method
     *
     *@return                       Description of the Return Value
     *@exception  LPParseException  Description of the Exception
     */
    final Literal headLiteral() throws LPParseException {
        Term[] args = null;
        String s = identifier();
        if (c == '(') {
            skip();
            args = parameterList();
            bracketRight();
        }
        return new Literal(s, args);
    }


    /**
     *  Description of the Method
     *
     *@return                       Description of the Return Value
     *@exception  LPParseException  Description of the Exception
     */
    final Literal[] headConjunction() throws LPParseException {
        ArrayList heads = new ArrayList();
        heads.add(headLiteral());
        while (c == '&') {
            skip();
            heads.add(headLiteral());
        }
        Literal[] a = new Literal[heads.size()];
        return (Literal[]) heads.toArray(a);
    }


    /**
     *  Description of the Method
     *
     *@return                       Description of the Return Value
     *@exception  LPParseException  Description of the Exception
     */
    final String identifier() throws LPParseException {
        if (!isLetter()) {
            throw new LPParseException("Line:" + is.getEndLine() + ".Column:" + is.getEndColumn() + ". Encountered  " + c + ". Letter expected");
        }
        while (isLetter() || isDigit()) {
            //System.out.print(c);
            try {
                c = is.readChar();
            } catch (java.io.IOException e) {
                throw new LPParseException("Line:" + is.getEndLine() + ".Column:" + is.getEndColumn() + ". Encountered  " + c + ". Unexpected End of File");
            }
        }
        is.backup(1);
        String s = is.GetImage();
        skip();
        return s;
    }


    /**
     *  Gets the digit attribute of the SimpleParser object
     *
     *@return    The digit value
     */
    final boolean isDigit() {
        return (('0' <= c) && ('9' >= c));
    }


    /**
     *  Gets the letter attribute of the SimpleParser object
     *
     *@return    The letter value
     */
    final boolean isLetter() {
        return ((('a' <= c) && (('z' >= c))) || (('A' <= c) && (('Z' >= c))) || (c == '_'));
    }


    /**
     *  Gets the sign attribute of the SimpleParser object
     *
     *@return    The sign value
     */
    final boolean isSign() {
        return ((c == '+') || (c == '-'));
    }


    /**
     *  Gets the uppercaseletter attribute of the SimpleParser object
     *
     *@return    The uppercaseletter value
     */
    final boolean isUppercaseletter() {
        return ((('A' <= c) && ('Z' >= c)) || (c == '_'));
    }


    /**
     *  Description of the Method
     *
     *@return                       Description of the Return Value
     *@exception  LPParseException  Description of the Exception
     */
    final Term[] parameterList() throws LPParseException {
        Term[] t;
        ArrayList param = new ArrayList(5);
        param.add(term());
        while (c == ',') {
            skip();
            param.add(term());
        }
        t = new Term[param.size()];
        return (Term[]) param.toArray(t);
    }


    /**
     *  Description of the Method
     *
     *@exception  LPParseException  Description of the Exception
     */
    final void dot() throws LPParseException {
        if (c != '.') {
            throw new LPParseException("Line:" + is.getEndLine() + ".Column:" + is.getEndColumn() + ". Encountered  " + c + ". Point expected");
        }
        skip();
    }


    /**
     *  Description of the Method
     *
     *@param  stream  Description of the Parameter
     */
    public void reInit(InputStream stream) {
        is.ReInit(stream, 1, 1);
    }


    /**
     *  Description of the Method
     *
     *@return                       Description of the Return Value
     *@exception  LPParseException  Description of the Exception
     */
    final ProgramClause parseRule() throws LPParseException {
        ProgramClause r;

        if (arrow()) {
            Literal[] body = bodyConjunction();
            r = new ProgramClause(null, body);
            dot();
        }
        else {
            Literal[] head = headConjunction();
            Literal[] body = null;
            if (arrow()) {
                body = bodyConjunction();
            }
            r = new ProgramClause(head, body);
            dot();
        }
        return r;
    }


    /**
     *  Description of the Method
     *
     *@exception  LPParseException  Description of the Exception
     */
    final void skip() throws LPParseException {
        try {
            c = is.BeginToken();
            while (((c == ' ') || (c == '\n') || (c == '\r'))) {
                c = is.readChar();
            }
            is.backup(1);
            c = is.BeginToken();
        } catch (java.io.IOException e) {
            fini = true;
        }
    }


    /**
     *  Description of the Method
     *
     *@return                       Description of the Return Value
     *@exception  LPParseException  Description of the Exception
     */
    final Term term() throws LPParseException {
        Term t = null;
        String s;
        int code;
        Term[] args;
        if (isUppercaseletter()) {
            return variable();
        }
        else if (isLetter()) {
            s = identifier();
            if (c == '(') {
                skip();
                args = parameterList();
                bracketRight();
                return new FunctorTerm(s, args);
            }
            else {
                return new ConstTerm(s);
            }
        }
        else if (isDigit() || isSign()) {
            return number();
        }
        else if (c == '"') {
            return new StringTerm(identifier());
        }
        else {
            throw new LPParseException("Line:" + is.getEndLine() + ".Column:" + is.getEndColumn() + ". Encountered  " + c + ". Variable, Constant, Function, Number or String expected");
        }
    }


    /**
     *  Description of the Method
     *
     *@return                       Description of the Return Value
     *@exception  LPParseException  Description of the Exception
     */
    final VariableTerm variable() throws LPParseException {
        return new VariableTerm(identifier());
    }


    /**
     *  Description of the Method
     *
     *@return                       Description of the Return Value
     *@exception  LPParseException  Description of the Exception
     */
    final DoubleTerm number() throws LPParseException {
        Double num;
        String s;
        if (isSign()) {
            try {
                c = is.readChar();
            } catch (java.io.IOException e) {
                throw new LPParseException("Line:" + is.getEndLine() + ".Column:" + is.getEndColumn() + ". Encountered  " + c + ". Unexpected End of File");
            }

            if (!isDigit()) {
                throw new LPParseException("Line:" + is.getEndLine() + ".Column:" + is.getEndColumn() + ". Encountered  " + c + ". digit expected");
            }
        }
        while (isDigit()) {
            try {
                c = is.readChar();
            } catch (java.io.IOException e) {
                throw new LPParseException("Line:" + is.getEndLine() + ".Column:" + is.getEndColumn() + ". Encountered  " + c + ". Unexpected End of File");
            }
        }
        if (c == '.') {
            try {
                c = is.readChar();
            } catch (java.io.IOException e) {
                throw new LPParseException("Line:" + is.getEndLine() + ".Column:" + is.getEndColumn() + ". Encountered  " + c + ". Unexpected End of File");
            }
            while (isDigit()) {
                try {
                    c = is.readChar();
                } catch (java.io.IOException e) {
                    throw new LPParseException("Line:" + is.getEndLine() + ".Column:" + is.getEndColumn() + ". Encountered  " + c + ". Unexpected End of File");
                }
            }
        }
        // Exponent
        if ((c == 'e') || (c == 'E')) {
            try {
                c = is.readChar();
            } catch (java.io.IOException e) {
                throw new LPParseException("Line:" + is.getEndLine() + ".Column:" + is.getEndColumn() + ". Encountered  " + c + ". Unexpected End of File");
            }
            if (!isSign() && !isDigit()) {
                throw new LPParseException("Line:" + is.getEndLine() + ".Column:" + is.getEndColumn() + ". Encountered  " + c + ". +,- or digit expected");
            }
            else {
                if (isSign()) {
                    try {
                        c = is.readChar();
                    } catch (java.io.IOException e) {
                        throw new LPParseException("Line:" + is.getEndLine() + ".Column:" + is.getEndColumn() + ". Encountered  " + c + ". Unexpected End of File");
                    }
                    if (!isDigit()) {
                        throw new LPParseException("Line:" + is.getEndLine() + ".Column:" + is.getEndColumn() + ". Encountered  " + c + ". digit expected");
                    }
                }
                while (isDigit()) {
                    try {
                        c = is.readChar();
                    } catch (java.io.IOException e) {
                        throw new LPParseException("Line:" + is.getEndLine() + ".Column:" + is.getEndColumn() + ". Encountered  " + c + ". Unexpected End of File");
                    }
                }
            }
        }
        is.backup(1);
        s = is.GetImage();
        try {
            num = new Double(s);
        } catch (NumberFormatException p) {
            throw new LPParseException("Line:" + is.getEndLine() + ".Column:" + is.getEndColumn() + ". Encountered  " + c + ". number expected");
        }
        skip();
        return new DoubleTerm(num.doubleValue());
    }


    /*
     *  final String identifier() throws SimpleParseException {
     *  String s;
     *  try {
     *  c = is.BeginToken();
     *  } catch (java.io.IOException e) {
     *  throw new SimpleParseException("Line:" + is.getEndLine() + ".Column:" + is.getEndColumn() + ". Encountered  " + c + ". Unexpected End of File");
     *  }
     *  while (c != '"') {
     *  try {
     *  c = is.readChar();
     *  } catch (java.io.IOException e) {
     *  throw new SimpleParseException("Line:" + is.getEndLine() + ".Column:" + is.getEndColumn() + ". Encountered  " + c + ". Unexpected End of File");
     *  }
     *  }
     *  is.backup(1);
     *  s = is.GetImage();
     *  try {
     *  c = is.readChar();
     *  } catch (java.io.IOException e) {
     *  throw new SimpleParseException("Line:" + is.getEndLine() + ".Column:" + is.getEndColumn() + ". Encountered  " + c + ". Unexpected End of File");
     *  }
     *  skip();
     *  return s;
     *  }
     */
    /**
     *  Description of the Class
     *
     *@author     Administrator
     *@created    February 6, 2002
     */
    static class ParserTester implements LPParserCallBack {
        /**
         *  Description of the Method
         *
         *@param  c  Description of the Parameter
         */
        public void call(ProgramClause c) {
            System.out.println("Parsed rule " + c);
        }
    }


    /**
     *  Description of the Method
     *
     *@param  argv  Description of the Parameter
     */
    public static void main(String argv[]) {
        String rules = "p(X,j(Y))<- p(o,X,j(O,P,L)). r(1.234E3).";
        LPParser sparser = new LPParser(new StringBufferInputStream(rules), new ParserTester());
        try {
            sparser.parse();
        } catch (Exception e) {
            System.out.println("Parse Exception " + e);
            e.printStackTrace();
        }
    }

}
