package org.semanticweb.triple.application;

import java.io.*;
import java.net.*;
import java.text.*;
import java.util.*;
import javax.servlet.*;
import javax.servlet.http.*;
import org.semanticweb.triple.parser.*;
import org.semanticweb.rdf.utilities.*;
import org.semanticweb.triple.language.*;


public class TripleAgent extends HttpServlet {


  // CONFIGURE HERE
  final static String xsbProgram = "/project/java/XSB/bin/xsb";
  final static String xsbMemory = "16000"; // kb
  final static String logDir = "/home/sintek/triple/tomcat/logs/triple/";
  // constants for rdf2java:
  final static String namespace = "http://www.w3.org/2001/06/30/triple#";
  final static String pkg = "dfki.rdf.triple.language";
  // END CONFIGURE


  public void doPost(HttpServletRequest request, 
	HttpServletResponse response) throws IOException, ServletException {
    doGet(request, response);
  }

  public void doGet(HttpServletRequest request,
        HttpServletResponse response) throws IOException, ServletException {
    transform(request, response);
  }

  // transform

  public void transform(HttpServletRequest request,
        HttpServletResponse response) throws IOException, ServletException {

    String language = request.getParameter("language");
    boolean asciiInput = "ASCII".equals(language);

    long timestamp = new Date().getTime();
    String tmpTripleFile = logDir + "triple" + timestamp;
    if (asciiInput)
      tmpTripleFile += ".triple";
    else
      tmpTripleFile += ".rdf";
    String tmpXSBFile = logDir + "triple" + timestamp + ".P";

    String tripleProgram = request.getParameter("program");
    writeFile(tmpTripleFile, tripleProgram);

    // create html result page
    response.setContentType("text/html");
    PrintWriter htmlWriter = response.getWriter();
    htmlWriter.println("<html>");
    htmlWriter.println("<body text=#000000 bgcolor=#DCDCDC>");

    TripleProgram pgm;
    if (asciiInput) {
      try {
        FileInputStream inputStream = new FileInputStream(tmpTripleFile);
	LTermParser parser = new LTermParser(inputStream);
	LTerm lpgm = parser.parse();
	inputStream.close();
	htmlWriter.println("<h2>Program as Hilog Term</h2>");
	htmlWriter.println("<pre>");
	htmlWriter.println(lpgm);
	htmlWriter.println("</pre>");
	pgm = new TripleProgram(lpgm);
      } catch (Exception e) {
	htmlWriter.println("error: " + e);
	pgm = null;
      }
    } else { // rdf input
      try {
	pgm = new TripleProgram(tmpTripleFile);
      } catch (Exception e) {
	htmlWriter.println("error: " + e);
	pgm = null;
      }
    }

    if (pgm != null) {
      // ASCII syntax
      htmlWriter.println("<h2>TRIPLE0/ASCII (flattened)</h2>");
      htmlWriter.println("<pre>");
      try {
	pgm.toASCII(htmlWriter);
      } catch (Exception e) {
	htmlWriter.println("error: " + e);
      }
      htmlWriter.println("</pre>");
      // Prolog program
      htmlWriter.println("<h2>XSB Prolog Program</h2>");
      htmlWriter.println("<pre>");
      try {
	pgm.toHornLogic(htmlWriter, false);
      } catch (Exception e) {
	htmlWriter.println("error: " + e);
      }
      htmlWriter.println("</pre>");
      // execute with XSB
      htmlWriter.println("<h2>Execution with XSB</h2>");
      try {
	PrintWriter writer = new PrintWriter(new FileWriter(tmpXSBFile));
	writer.println(":- auto_table.");
	pgm.toHornLogic(writer, true);
	writer.flush();
	writer.close();
	String[] xsbCommand = new String[] { xsbProgram, "--noprompt", 
	  "-m", xsbMemory, "-e", "['" + tmpXSBFile + "']." };
	String xsbOutput = runExternalProgram(xsbCommand, "halt.");
	htmlWriter.println("<pre>");
	htmlWriter.println(xsbOutput);
	htmlWriter.println("</pre>");
	writer.flush();
	writer.close();
      } catch (Exception e) { System.err.println(e); }
    }
    htmlWriter.println("</body>");
    htmlWriter.println("</html>");

  }


  // auxiliaries

  void writeFile(String fileName, String contents) {
    try {
      PrintWriter writer = new PrintWriter(new FileWriter(fileName));
      if (contents != null)
	writer.println(contents);
      writer.flush();
      writer.close();
    } catch (Exception e) { System.err.println(e); }
  }

  String runExternalProgram(String[] command, String sendString) 
      throws Exception {
    Process process = Runtime.getRuntime().exec(command);
    PrintWriter outputStream = new PrintWriter(process.getOutputStream(), true);
    BufferedReader reader = 
      new BufferedReader(new InputStreamReader(process.getInputStream()));
    if (sendString != null)
      outputStream.println(sendString);
    // outputStream.close(); // ???
    StringBuffer buffer = new StringBuffer();
    String line = reader.readLine();
    while (line != null) {
      buffer.append(line);
      buffer.append("\n");
      line = reader.readLine();
    }
    return buffer.toString();
  }

}


