/*
 * Commandline.java
 *
 * Created on February 8, 2002, 9:57 AM
 */

package org.semanticweb.triple.application;

import java.io.*;
import org.semanticweb.triple.parser.*;

/**
 * Process TRIPLE programs from command line.
 * @author  Michael Sintek
 * @version
 */
public class Commandline {

  public static void main(String args[]) {
    LTermParser parser = null;
    String inputFileName = null;
    if (args.length == 0) {
      parser = new LTermParser(System.in);
      inputFileName = "standard input";
    } else { // process arguments ... (output filename etc.) !!!
      try {
        inputFileName = args[0];
        parser = new LTermParser(new java.io.FileInputStream(inputFileName));
      } catch (java.io.FileNotFoundException e) {
        System.out.println("Error: file '" + inputFileName + "' not found.");
        System.exit(1);
      }
    }
    try {
      String fileName = "out.P";
      PrintWriter writer = new PrintWriter(new FileWriter(fileName));
      writer.println("% " + fileName + ", generated from " 
                     + inputFileName + "\n");
      writer.println(":-table(true/2).\n");
      parser.parse(new Context(new ExternalPrologCallback(writer)));
      writer.println(":- halt.\n");
      writer.println("% auxiliaries\n");
      writer.println("write_triple(true(S,M)) :- !, write_triple(S), write('@'), write_triple(M).");
      writer.println("write_triple(resource(NS,LN)) :- !, write_triple(NS), write(':'), write_triple(LN).");
      writer.println("write_triple(triple(S,P,O)) :- !, write_triple(S), write('['), write_triple(P), write(','), write_triple(O), write(']').");
      writer.println("write_triple((T1,T2)) :- !, write_triple(T1), write(', '), write_triple(T2).");
      writer.println("write_triple(T) :- writeq(T).");
      writer.close();
      System.out.println("'" + fileName + "' written.");
    } catch (Exception e) {
      System.out.println("Error: " + e.getMessage());
      System.exit(1);
    }
  }

}
