/*
 * ExternalPrologCallback.java
 *
 * Created on February 9, 2002, 9:35 AM
 */

package org.semanticweb.triple.application;

import java.util.*;
import java.io.*;
import org.semanticweb.triple.parser.*;
import org.semanticweb.triple.language.Clause;
import org.semanticweb.glp.*;
import org.semanticweb.lp.*;


/**
 * Implementation of Callback to generate output usable for
 * external Prologs like XSB.
 * @author  Michael Sintek
 * @version 
 */
public class ExternalPrologCallback implements Callback {

  PrintWriter writer;
  
  public ExternalPrologCallback() {
    this.writer = new PrintWriter(System.out);
  }
  
  public ExternalPrologCallback(PrintWriter writer) {
    this.writer = writer;
  }
  
  public void nextClause(LTerm lclause, Context context) {
    LTerm lmodel = context.getModel();
    // writer.println("% HIL: " + lclause + " @ " + lmodel); // hilog representation
    try {
      // 1. native TRIPLE representation
      Clause clause = TripleTransform.transformClause(lclause, lmodel);
      writer.println("% TRP: " + clause);
      // 2. GLP
      GLPRule glpClause = (GLPRule)clause.toGLP(); // GLPClause ??
      writer.println("% GLP: " + glpClause);
      // 3. LP (Lloyd Topor application)
      GLPLloydToporTransform lt = new GLPLloydToporTransform();
      ArrayList lpRules = lt.translateFormula((GLPRule)glpClause);
      // 4. print as PROLOG program
      ArrayList answerVars = new ArrayList(); // of lp.VariableTerm
      ArrayList glpVars = glpClause.getVariables();
      if (glpVars != null) {
        for (Iterator varIter = glpVars.iterator(); varIter.hasNext();) {
          answerVars.add(((GLPTermVariable)varIter.next()).toLPTerm());
        }
      }
      for (Iterator i = lpRules.iterator(); i.hasNext();) {
        ProgramClause lpClause = (ProgramClause)i.next();
        writer.println(lpClause.toPrologString(answerVars));
      }
    } catch (Exception e) {
      writer.println("% ???: " + e);
      e.printStackTrace();
    }
    writer.println();
  }
    
}
