package org.semanticweb.triple.language;

import java.util.*;
import org.semanticweb.glp.*;


public class HornAtom extends Atom {

  String predicateSymbol;
  ArrayList args = new ArrayList(); // of Term

  public void putPredicateSymbol(String predicateSymbol) { 
    this.predicateSymbol = predicateSymbol; 
  }

  public void putArgs(Term term) { args.add(term); }
  public void putArgs(String name) { args.add(new Literal(name)); }

  public String getPredicateSymbol() { return predicateSymbol; }
  public ArrayList getArgs() { return args; }

  public String toString(Syntax syntax) {
    StringBuffer buffer = new StringBuffer();
    buffer.append(syntax.getSymbol(predicateSymbol));
    buffer.append("(");
    if (args != null && args.size() > 0) {
      for (Iterator termIterator = args.iterator();
	   termIterator.hasNext();) {
	Term term = (Term)termIterator.next();
	buffer.append(term.toString(syntax));
	if (termIterator.hasNext())
	  buffer.append(", ");
      }
    }
    buffer.append(")");
    appendModel(syntax, buffer);
    return buffer.toString();
  }
  
  public GLogicFormula toGLP(GLPTerm model) {
    // p(a1, ...) -> ..., true(p(a1', ...), model)
    // this -> triple struct
    ArrayList additionalGlpFormulae = new ArrayList();
    model = overrideModel(model, this.model, additionalGlpFormulae);
    ArrayList structargs = new ArrayList(3);
    for (Iterator argIterator = args.iterator();
	 argIterator.hasNext();) {
      Term arg = (Term)argIterator.next();
      GLPTerm newArg = (GLPTerm)arg.toGLP(model, additionalGlpFormulae);
      structargs.add(newArg);
    }
    GLPTermFunction struct = 
      new GLPTermFunction(predicateSymbol, structargs);
    ArrayList trueargs = new ArrayList(2);
    trueargs.add(struct);
    trueargs.add(model);
    GLPAtom trueAtom = new GLPAtom("true", trueargs);
    GLogicFormula conjunction =
      And.makeGLPConjunction(additionalGlpFormulae, trueAtom);
    return conjunction;
  }

}
